@@ -40,7 +40,6 @@
 #include <linux/videodev.h>
 
 #include "cheese-webcam.h"
-#include "cheese-flash.h"
 
 G_DEFINE_TYPE (CheeseWebcam, cheese_webcam, G_TYPE_OBJECT)
 
@@ -48,6 +47,8 @@ G_DEFINE_TYPE (CheeseWebcam, cheese_webcam, G_TYPE_OBJECT)
 
 #define CHEESE_WEBCAM_ERROR cheese_webcam_error_quark ()
 
+#define MIN_DEFAULT_RATE 15.0
+
 static void find_highest_framerate (CheeseVideoFormat *format);
 
 enum CheeseWebcamError
@@ -78,6 +79,7 @@ typedef struct
   GstElement *video_enc;
 
   GstElement *effect_filter, *csp_post_effect;
+  GstElement *video_balance, *csp_post_balance;
 
   gulong photo_handler_signal_id;
 
@@ -93,8 +95,6 @@ typedef struct
   int selected_device;
   CheeseVideoFormat *current_format;
 
-  CheeseFlash *flash;
-
   guint eos_timeout_id;
 } CheeseWebcamPrivate;
 
@@ -153,15 +153,30 @@ cheese_webcam_error_quark (void)
   return g_quark_from_static_string ("cheese-webcam-error-quark");
 }
 
-static void
-cheese_webcam_set_x_overlay (CheeseWebcam *webcam)
+static GstBusSyncReply
+cheese_webcam_bus_sync_handler (GstBus *bus, GstMessage *message, CheeseWebcam *webcam)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  GstXOverlay         *overlay;
 
-  GstXOverlay *overlay = GST_X_OVERLAY (gst_bin_get_by_interface (GST_BIN (priv->pipeline),
-                                                                  GST_TYPE_X_OVERLAY));
+  if (GST_MESSAGE_TYPE (message) != GST_MESSAGE_ELEMENT)
+    return GST_BUS_PASS;
 
-  gst_x_overlay_set_xwindow_id (overlay, GDK_WINDOW_XWINDOW (priv->video_window->window));
+  if (!gst_structure_has_name (message->structure, "prepare-xwindow-id"))
+    return GST_BUS_PASS;
+
+  overlay = GST_X_OVERLAY (GST_MESSAGE_SRC (message));
+
+  if (g_object_class_find_property (G_OBJECT_GET_CLASS (overlay),
+                                    "force-aspect-ratio"))
+    g_object_set (G_OBJECT (overlay), "force-aspect-ratio", TRUE, NULL);
+
+  gst_x_overlay_set_xwindow_id (overlay,
+                                GDK_WINDOW_XWINDOW (gtk_widget_get_window (priv->video_window)));
+
+  gst_message_unref (message);
+
+  return GST_BUS_DROP;
 }
 
 static void
@@ -187,7 +202,23 @@ cheese_webcam_change_sink (CheeseWebcam *webcam, GstElement *src,
 static gboolean
 cheese_webcam_expose_cb (GtkWidget *widget, GdkEventExpose *event, CheeseWebcam *webcam)
 {
-  cheese_webcam_set_x_overlay (webcam);
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  GstState             state;
+  GstXOverlay         *overlay = GST_X_OVERLAY (gst_bin_get_by_interface (GST_BIN (priv->pipeline),
+                                                                          GST_TYPE_X_OVERLAY));
+
+  gst_element_get_state (priv->pipeline, &state, NULL, 0);
+
+  if ((state < GST_STATE_PLAYING) || (overlay == NULL))
+  {
+    gdk_draw_rectangle (widget->window, widget->style->black_gc, TRUE,
+                        0, 0, widget->allocation.width, widget->allocation.height);
+  }
+  else
+  {
+    gst_x_overlay_expose (overlay);
+  }
+
   return FALSE;
 }
 
@@ -218,6 +249,7 @@ cheese_webcam_photo_data_cb (GstElement *element, GstBuffer *buffer,
 
   g_signal_handler_disconnect (G_OBJECT (priv->photo_sink),
                                priv->photo_handler_signal_id);
+  priv->photo_handler_signal_id = 0;
 
   g_signal_emit (webcam, webcam_signals[PHOTO_SAVED], 0);
 }
@@ -229,7 +261,8 @@ cheese_webcam_bus_message_cb (GstBus *bus, GstMessage *message, CheeseWebcam *we
 
   if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_EOS)
   {
-    if (priv->is_recording) {
+    if (priv->is_recording)
+    {
       g_print ("Received EOS message\n");
 
       g_source_remove (priv->eos_timeout_id);
@@ -302,13 +335,13 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
   {
     char                   *device;
     char                   *parent_udi = NULL;
-    char                   *subsystem = NULL;
+    char                   *subsystem  = NULL;
     char                   *gstreamer_src, *product_name;
     struct v4l2_capability  v2cap;
     struct video_capability v1cap;
-    gint vendor_id = 0;
-    gint product_id = 0;
-    gchar *property_name = NULL;
+    gint                    vendor_id     = 0;
+    gint                    product_id    = 0;
+    gchar                  *property_name = NULL;
 
     parent_udi = libhal_device_get_property_string (hal_ctx, udis[i], "info.parent", &error);
     if (dbus_error_is_set (&error))
@@ -317,20 +350,23 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
       dbus_error_free (&error);
     }
 
-    if (parent_udi != NULL) {
+    if (parent_udi != NULL)
+    {
       subsystem = libhal_device_get_property_string (hal_ctx, parent_udi, "info.subsystem", NULL);
       if (subsystem == NULL) continue;
       property_name = g_strjoin (".", subsystem, "vendor_id", NULL);
-      vendor_id = libhal_device_get_property_int (hal_ctx, parent_udi, property_name , &error);
-      if (dbus_error_is_set (&error)) {
+      vendor_id     = libhal_device_get_property_int (hal_ctx, parent_udi, property_name, &error);
+      if (dbus_error_is_set (&error))
+      {
         g_warning ("error getting vendor id: %s: %s", error.name, error.message);
         dbus_error_free (&error);
       }
       g_free (property_name);
 
       property_name = g_strjoin (".", subsystem, "product_id", NULL);
-      product_id = libhal_device_get_property_int (hal_ctx, parent_udi, property_name, &error);
-      if (dbus_error_is_set (&error)) {
+      product_id    = libhal_device_get_property_int (hal_ctx, parent_udi, property_name, &error);
+      if (dbus_error_is_set (&error))
+      {
         g_warning ("error getting product id: %s: %s", error.name, error.message);
         dbus_error_free (&error);
       }
@@ -371,7 +407,7 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
       if (ok < 0)
       {
         g_warning ("Error while probing v4l capabilities for %s: %s",
-                 device, strerror (errno));
+                   device, strerror (errno));
         libhal_free_string (device);
         close (fd);
         continue;
@@ -386,8 +422,9 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
       guint cap = v2cap.capabilities;
       g_print ("Detected v4l2 device: %s\n", v2cap.card);
       g_print ("Driver: %s, version: %d\n", v2cap.driver, v2cap.version);
+
       /* g_print ("Bus info: %s\n", v2cap.bus_info); */ /* Doesn't seem anything useful */
-     g_print ("Capabilities: 0x%08X\n", v2cap.capabilities);
+      g_print ("Capabilities: 0x%08X\n", v2cap.capabilities);
       if (!(cap & V4L2_CAP_VIDEO_CAPTURE))
       {
         g_print ("Device %s seems to not have the capture capability, (radio tuner?)\n"
@@ -496,36 +533,38 @@ cheese_webcam_get_supported_framerates (CheeseVideoFormat *video_format, GstStru
 
 static void
 cheese_webcam_add_video_format (CheeseWebcamDevice *webcam_device,
-  CheeseVideoFormat *video_format, GstStructure *format_structure)
+                                CheeseVideoFormat *video_format, GstStructure *format_structure)
 {
-  int i;
+  int    i;
   gchar *resolution;
 
   cheese_webcam_get_supported_framerates (video_format, format_structure);
   find_highest_framerate (video_format);
 
-  g_print ("%s %d x %d num_framerates %d\n", video_format->mimetype, video_format->width, 
+  g_print ("%s %d x %d num_framerates %d\n", video_format->mimetype, video_format->width,
            video_format->height, video_format->num_framerates);
   for (i = 0; i < video_format->num_framerates; i++)
   {
     g_print ("%d/%d ", video_format->framerates[i].numerator,
              video_format->framerates[i].denominator);
   }
-  
+
   resolution = g_strdup_printf ("%ix%i", video_format->width,
                                 video_format->height);
-  i = GPOINTER_TO_INT(g_hash_table_lookup (
-                                     webcam_device->supported_resolutions,
-                                     resolution));
-  if (i) { /* Resolution already added ? */
-    CheeseVideoFormat *curr_format = &g_array_index(
-                                                webcam_device->video_formats,
-                                                CheeseVideoFormat, i - 1);
-    float new_framerate = (float)video_format->highest_framerate.numerator /
-                                 video_format->highest_framerate.denominator;
-    float curr_framerate = (float)curr_format->highest_framerate.numerator /
-                                  curr_format->highest_framerate.denominator;
-    if (new_framerate > curr_framerate) {
+  i = GPOINTER_TO_INT (g_hash_table_lookup (
+                         webcam_device->supported_resolutions,
+                         resolution));
+  if (i)   /* Resolution already added ? */
+  {
+    CheeseVideoFormat *curr_format = &g_array_index (
+      webcam_device->video_formats,
+      CheeseVideoFormat, i - 1);
+    float new_framerate = (float) video_format->highest_framerate.numerator /
+                          video_format->highest_framerate.denominator;
+    float curr_framerate = (float) curr_format->highest_framerate.numerator /
+                           curr_format->highest_framerate.denominator;
+    if (new_framerate > curr_framerate)
+    {
       g_print ("higher framerate replacing existing format\n");
       *curr_format = *video_format;
     }
@@ -538,12 +577,13 @@ cheese_webcam_add_video_format (CheeseWebcamDevice *webcam_device,
 
   g_array_append_val (webcam_device->video_formats, *video_format);
   g_hash_table_insert (webcam_device->supported_resolutions, resolution,
-                       GINT_TO_POINTER(webcam_device->num_video_formats + 1));
+                       GINT_TO_POINTER (webcam_device->num_video_formats + 1));
 
   webcam_device->num_video_formats++;
 }
 
-static gint cheese_resolution_compare(gconstpointer _a, gconstpointer _b)
+static gint
+cheese_resolution_compare (gconstpointer _a, gconstpointer _b)
 {
   const CheeseVideoFormat *a = _a;
   const CheeseVideoFormat *b = _b;
@@ -587,7 +627,7 @@ cheese_webcam_get_supported_video_formats (CheeseWebcamDevice *webcam_device, Gs
       video_format.mimetype = g_strdup (gst_structure_get_name (structure));
       gst_structure_get_int (structure, "width", &(video_format.width));
       gst_structure_get_int (structure, "height", &(video_format.height));
-      cheese_webcam_add_video_format(webcam_device, &video_format, structure);
+      cheese_webcam_add_video_format (webcam_device, &video_format, structure);
     }
     else if (GST_VALUE_HOLDS_INT_RANGE (width))
     {
@@ -601,8 +641,9 @@ cheese_webcam_get_supported_video_formats (CheeseWebcamDevice *webcam_device, Gs
 
       cur_width  = min_width;
       cur_height = min_height;
+
       /* Gstreamer will sometimes give us a range with min_xxx == max_xxx,
-         we use <= here (and not below) to make this work */
+       * we use <= here (and not below) to make this work */
       while (cur_width <= max_width && cur_height <= max_height)
       {
         CheeseVideoFormat video_format;
@@ -610,7 +651,7 @@ cheese_webcam_get_supported_video_formats (CheeseWebcamDevice *webcam_device, Gs
         video_format.mimetype = g_strdup (gst_structure_get_name (structure));
         video_format.width    = cur_width;
         video_format.height   = cur_height;
-        cheese_webcam_add_video_format(webcam_device, &video_format, structure);
+        cheese_webcam_add_video_format (webcam_device, &video_format, structure);
         cur_width  *= 2;
         cur_height *= 2;
       }
@@ -624,7 +665,7 @@ cheese_webcam_get_supported_video_formats (CheeseWebcamDevice *webcam_device, Gs
         video_format.mimetype = g_strdup (gst_structure_get_name (structure));
         video_format.width    = cur_width;
         video_format.height   = cur_height;
-        cheese_webcam_add_video_format(webcam_device, &video_format, structure);
+        cheese_webcam_add_video_format (webcam_device, &video_format, structure);
         cur_width  /= 2;
         cur_height /= 2;
       }
@@ -634,19 +675,20 @@ cheese_webcam_get_supported_video_formats (CheeseWebcamDevice *webcam_device, Gs
       g_critical ("GValue type %s, cannot be handled for resolution width", G_VALUE_TYPE_NAME (width));
     }
   }
-  
+
   /* Sort the format array (so that it will show sorted in the resolution
-     selection GUI), and rebuild the hashtable (as that will be invalid after
-     the sorting) */
+   * selection GUI), and rebuild the hashtable (as that will be invalid after
+   * the sorting) */
   g_array_sort (webcam_device->video_formats, cheese_resolution_compare);
   g_hash_table_remove_all (webcam_device->supported_resolutions);
-  for (i = 0; i < webcam_device->num_video_formats; i++) {
-    CheeseVideoFormat *format = &g_array_index(webcam_device->video_formats,
-                                               CheeseVideoFormat, i);
-    g_hash_table_insert (webcam_device->supported_resolutions, 
+  for (i = 0; i < webcam_device->num_video_formats; i++)
+  {
+    CheeseVideoFormat *format = &g_array_index (webcam_device->video_formats,
+                                                CheeseVideoFormat, i);
+    g_hash_table_insert (webcam_device->supported_resolutions,
                          g_strdup_printf ("%ix%i", format->width,
-                                          format->height), 
-                         GINT_TO_POINTER(i + 1));
+                                          format->height),
+                         GINT_TO_POINTER (i + 1));
   }
 }
 
@@ -772,7 +814,7 @@ find_highest_framerate (CheeseVideoFormat *format)
     }
   }
 
-  format->highest_framerate.numerator = framerate_numerator;
+  format->highest_framerate.numerator   = framerate_numerator;
   format->highest_framerate.denominator = framerate_denominator;
 }
 
@@ -814,7 +856,7 @@ cheese_webcam_create_webcam_source_bin (CheeseWebcam *webcam)
      * camera supports it. */
     if (priv->x_resolution != 0 && priv->y_resolution != 0)
     {
-      i = GPOINTER_TO_INT(g_hash_table_lookup (selected_webcam->supported_resolutions, resolution));
+      i = GPOINTER_TO_INT (g_hash_table_lookup (selected_webcam->supported_resolutions, resolution));
       if (i)
         format = &g_array_index (selected_webcam->video_formats,
                                  CheeseVideoFormat, i - 1);
@@ -827,11 +869,14 @@ cheese_webcam_create_webcam_source_bin (CheeseWebcam *webcam)
                                 CheeseVideoFormat, 0));
       for (i = 1; i < selected_webcam->num_video_formats; i++)
       {
-        if (g_array_index (selected_webcam->video_formats,
-                           CheeseVideoFormat, i).width > format->width)
+        CheeseVideoFormat *new = &g_array_index (selected_webcam->video_formats,
+                                                 CheeseVideoFormat, i);
+        gfloat newrate = new->highest_framerate.numerator /
+                         new->highest_framerate.denominator;
+        if ((new->width + new->height) > (format->width + format->height) &&
+            (newrate >= MIN_DEFAULT_RATE))
         {
-          format = &(g_array_index (selected_webcam->video_formats,
-                                    CheeseVideoFormat, i));
+          format = new;
         }
       }
     }
@@ -943,6 +988,16 @@ cheese_webcam_create_video_display_bin (CheeseWebcam *webcam, GError **error)
   {
     cheese_webcam_set_error_element_not_found (error, "ffmpegcolorspace");
   }
+  if ((priv->video_balance = gst_element_factory_make ("videobalance", "video_balance")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found (error, "videobalance");
+    return FALSE;
+  }
+  if ((priv->csp_post_balance = gst_element_factory_make ("ffmpegcolorspace", "csp_post_balance")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found (error, "ffmpegcolorspace");
+    return FALSE;
+  }
 
   if ((tee = gst_element_factory_make ("tee", "tee")) == NULL)
   {
@@ -978,11 +1033,15 @@ cheese_webcam_create_video_display_bin (CheeseWebcam *webcam, GError **error)
     return FALSE;
 
   gst_bin_add_many (GST_BIN (priv->video_display_bin), priv->webcam_source_bin,
-                    priv->effect_filter, priv->csp_post_effect, tee, save_queue,
+                    priv->effect_filter, priv->csp_post_effect,
+                    priv->video_balance, priv->csp_post_balance,
+                    tee, save_queue,
                     video_display_queue, video_scale, video_sink, NULL);
 
   ok = gst_element_link_many (priv->webcam_source_bin, priv->effect_filter,
-                              priv->csp_post_effect, tee, NULL);
+                              priv->csp_post_effect,
+                              priv->video_balance, priv->csp_post_balance,
+                              tee, NULL);
 
   ok &= gst_element_link_many (tee, save_queue, NULL);
   ok &= gst_element_link_many (tee, video_display_queue, video_scale, video_sink, NULL);
@@ -1108,6 +1167,13 @@ cheese_webcam_create_video_save_bin (CheeseWebcam *webcam, GError **error)
   {
     cheese_webcam_set_error_element_not_found (error, "oggmux");
   }
+  else
+  {
+    g_object_set (G_OBJECT (mux),
+                  "max-delay", (guint64) 10000000,
+                  "max-page-delay", (guint64) 10000000, NULL);
+  }
+
   if ((priv->video_file_sink = gst_element_factory_make ("filesink", "video_file_sink")) == NULL)
   {
     cheese_webcam_set_error_element_not_found (error, "filesink");
@@ -1210,11 +1276,7 @@ cheese_webcam_play (CheeseWebcam *webcam)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
-  GstStateChangeReturn ret;
-
-  ret = gst_element_set_state (priv->pipeline, GST_STATE_PLAYING);
-  cheese_webcam_set_x_overlay (webcam);
-
+  gst_element_set_state (priv->pipeline, GST_STATE_PLAYING);
   priv->pipeline_is_playing = TRUE;
 }
 
@@ -1312,10 +1374,11 @@ cheese_webcam_start_video_recording (CheeseWebcam *webcam, char *filename)
 static gboolean
 cheese_webcam_force_stop_video_recording (gpointer data)
 {
-  CheeseWebcam *webcam = CHEESE_WEBCAM (data);
-  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  CheeseWebcam        *webcam = CHEESE_WEBCAM (data);
+  CheeseWebcamPrivate *priv   = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
-  if (priv->is_recording) {
+  if (priv->is_recording)
+  {
     g_print ("Cannot cleanly shutdown recording pipeline, forcing\n");
     g_signal_emit (webcam, webcam_signals[VIDEO_SAVED], 0);
 
@@ -1331,26 +1394,35 @@ void
 cheese_webcam_stop_video_recording (CheeseWebcam *webcam)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
-  GstState state;
+  GstState             state;
 
   gst_element_get_state (priv->pipeline, &state, NULL, 0);
 
-  if (state == GST_STATE_PLAYING) {
+  if (state == GST_STATE_PLAYING)
+  {
     /* Send EOS message down the pipeline by stopping video and audio source*/
     g_print ("Sending EOS event down the recording pipeline\n");
     gst_element_send_event (priv->video_source, gst_event_new_eos ());
     gst_element_send_event (priv->audio_source, gst_event_new_eos ());
     priv->eos_timeout_id = g_timeout_add (3000, cheese_webcam_force_stop_video_recording, webcam);
-  } else {
+  }
+  else
+  {
     cheese_webcam_force_stop_video_recording (webcam);
   }
 }
 
-void
+gboolean
 cheese_webcam_take_photo (CheeseWebcam *webcam, char *filename)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
+  if (priv->photo_handler_signal_id != 0)
+  {
+    g_print ("Still waiting for previous photo data, ignoring new request\n");
+    return FALSE;
+  }
+
   g_free (priv->photo_filename);
   priv->photo_filename = g_strdup (filename);
 
@@ -1359,7 +1431,7 @@ cheese_webcam_take_photo (CheeseWebcam *webcam, char *filename)
                                                     "handoff",
                                                     G_CALLBACK (cheese_webcam_photo_data_cb),
                                                     webcam);
-  cheese_flash_fire (priv->flash);
+  return TRUE;
 }
 
 static void
@@ -1529,13 +1601,12 @@ cheese_webcam_init (CheeseWebcam *webcam)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
-  priv->is_recording        = FALSE;
-  priv->pipeline_is_playing = FALSE;
-  priv->photo_filename      = NULL;
-  priv->webcam_devices      = NULL;
-  priv->device_name         = NULL;
-
-  priv->flash = cheese_flash_new ();
+  priv->is_recording            = FALSE;
+  priv->pipeline_is_playing     = FALSE;
+  priv->photo_filename          = NULL;
+  priv->webcam_devices          = NULL;
+  priv->device_name             = NULL;
+  priv->photo_handler_signal_id = 0;
 }
 
 CheeseWebcam *
@@ -1601,6 +1672,8 @@ cheese_webcam_setup (CheeseWebcam *webcam, char *hal_dev_udi, GError **error)
   g_signal_connect (G_OBJECT (priv->bus), "message",
                     G_CALLBACK (cheese_webcam_bus_message_cb), webcam);
 
+  gst_bus_set_sync_handler (priv->bus, (GstBusSyncHandler) cheese_webcam_bus_sync_handler, webcam);
+
   if (!ok)
     g_error ("Unable link pipeline for photo");
 }
@@ -1702,3 +1775,33 @@ cheese_webcam_get_current_video_format (CheeseWebcam *webcam)
 
   return priv->current_format;
 }
+
+void
+cheese_webcam_get_balance_property_range (CheeseWebcam *webcam,
+                                          gchar *property,
+                                          gdouble *min, gdouble *max, gdouble *def)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  GParamSpec          *pspec;
+
+  *min = 0.0;
+  *max = 0.0;
+  *def = 0.0;
+
+  pspec = g_object_class_find_property (
+    G_OBJECT_GET_CLASS (G_OBJECT (priv->video_balance)), property);
+
+  g_return_if_fail (pspec != NULL);
+
+  *min = G_PARAM_SPEC_DOUBLE (pspec)->minimum;
+  *max = G_PARAM_SPEC_DOUBLE (pspec)->maximum;
+  *def = G_PARAM_SPEC_DOUBLE (pspec)->default_value;
+}
+
+void
+cheese_webcam_set_balance_property (CheeseWebcam *webcam, gchar *property, gdouble value)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+
+  g_object_set (G_OBJECT (priv->video_balance), property, value, NULL);
+}
